<?php
# Name: ebl_article_image_modal
# Type: admin
# Author: Eric B. Limegrover
# Version 1.0

if (!defined('txpinterface')) {
    exit;
}

if (txpinterface === 'admin') {
    new ebl_article_image_modal();
}

class ebl_article_image_modal
{
    protected $fetch_event = 'eblArticleImageFetch';

    public function __construct()
    {
        // Grant Publishers (level 1) access
        add_privs('article', '1');
        add_privs($this->fetch_event, '1');

        // Inject the dialog markup & JS into the Article panel
        register_callback([$this, 'inject_modal_script'], 'article');

        // Handle AJAX fetching of images
        register_callback([$this, 'fetch_images'], $this->fetch_event);
    }

    public function inject_modal_script()
    {

        echo <<<HTML
<style>
/* Center the dialog in the viewport */
#article-image-modal.ui-dialog {
    position: fixed; 
    top: 50%; 
    left: 50%; 
    transform: translate(-50%, -50%);

}

/* Make sure the dialog has enough z-index to appear over everything */
#article-image-modal.ui-dialog {
    z-index: 9999;
}

</style>

<script>
document.addEventListener('DOMContentLoaded', function() {
    const articleImageInput = document.getElementById('article-image');
    if (!articleImageInput) {
        return;
    }

    // Hide the original input
    articleImageInput.type = 'hidden';

    // Create "Select Image" link
    const selectLink = document.createElement('a');
    selectLink.href = '#';
    selectLink.textContent = 'Select Image';
    selectLink.style.marginLeft = '10px';
    articleImageInput.parentNode.insertBefore(selectLink, articleImageInput.nextSibling);

    // Preview container for chosen image
    const selectedPreview = document.createElement('div');
    selectedPreview.id = 'selected-image-preview';
    selectedPreview.style.marginTop = '10px';
    selectLink.parentNode.insertBefore(selectedPreview, selectLink.nextSibling);

    /**
     * Displays the chosen thumbnail + a "Clear Image" button
     */
    function displaySelectedThumbnail(imgObj) {
        if (!imgObj || !imgObj.thumbnail_url) {
            selectedPreview.innerHTML = '';
            return;
        }
        selectedPreview.innerHTML = [
            '<div style="display:inline-flex; align-items:center; gap:8px;">',
                '<img src="', imgObj.thumbnail_url, '"',
                     ' alt="', (imgObj.alt || ''), '"',
                     ' title="', (imgObj.name || ''), '"',
                     ' style="max-width:75px; height:auto; border:1px solid #ccc; border-radius:3px;" />',
                '<button class="clear-selected-image" title="Clear selected image">',
                    'Clear Image',
                '</button>',
            '</div>'
        ].join('');

        const clearBtn = selectedPreview.querySelector('.clear-selected-image');
        clearBtn.addEventListener('click', function() {
            articleImageInput.value = '';
            selectedPreview.innerHTML = '';
        });
    }

    // If there's an existing image ID, fetch it to show on load
    if (articleImageInput.value) {
        fetch('index.php?event={$this->fetch_event}&action=single&id=' + encodeURIComponent(articleImageInput.value))
            .then(r => r.text())
            .then(text => {
                try {
                    return JSON.parse(text);
                } catch (err) {
                    throw new Error('Single-image parse error: ' + err.message);
                }
            })
            .then(data => {
                if (data && !data.error) {
                    displaySelectedThumbnail(data);
                }
            })
            .catch(err => console.error('Error fetching single image:', err));
    }

    // Dialog references
    const modal     = document.getElementById('article-image-modal');
    const closeBtn  = document.getElementById('close-article-image-modal');
    const container = document.getElementById('thumbnail-container');
    const prevBtn   = document.getElementById('prev-btn');
    const nextBtn   = document.getElementById('next-btn');

    let overlay = null;

    // Show the dialog + create .ui-overlay
    function showDialog() {
        overlay = document.createElement('div');
        overlay.className = 'ui-overlay ui-widget-overlay';
        document.body.appendChild(overlay);

        // Display the dialog
        modal.style.display = 'block';

        // ESC to close
        document.addEventListener('keydown', escClose);
    }

    // Hide the dialog & remove overlay
    function hideDialog() {
        modal.style.display = 'none';
        if (overlay && overlay.parentNode) {
            overlay.parentNode.removeChild(overlay);
            overlay = null;
        }
        document.removeEventListener('keydown', escClose);
    }

    function escClose(e) {
        if (e.key === 'Escape') {
            hideDialog();
        }
    }

    // Pagination
    let imagesData  = [];
    let currentPage = 0;
    const perPage   = 10;
    let totalPages  = 0;

    function showPage() {
        container.innerHTML = '';
        const start = currentPage * perPage;
        const end   = start + perPage;
        const chunk = imagesData.slice(start, end);

        chunk.forEach(img => {
            const thumb = document.createElement('img');
            thumb.src   = img.thumbnail_url;
            thumb.alt   = img.alt || 'Image';
            thumb.title = img.name || 'Image';
            thumb.style.cursor       = 'pointer';
            thumb.style.margin       = '5px';
            thumb.style.maxWidth     = '75px';
            thumb.style.border       = '1px solid #ccc';
            thumb.style.borderRadius = '3px';

            thumb.addEventListener('click', function() {
                articleImageInput.value = img.id;
                displaySelectedThumbnail(img);
                hideDialog();
            });
            container.appendChild(thumb);
        });

        // Show/hide prev/next
        prevBtn.style.display = (currentPage <= 0) ? 'none' : '';
        nextBtn.style.display = (currentPage >= totalPages - 1) ? 'none' : '';
    }

    function loadAllImages() {
        fetch('index.php?event={$this->fetch_event}&action=all')
            .then(r => r.text())
            .then(text => {
                try {
                    return JSON.parse(text);
                } catch (err) {
                    throw new Error('Images JSON parse error: ' + err.message);
                }
            })
            .then(data => {
                if (!Array.isArray(data)) {
                    container.innerHTML = '<p>Unexpected response format.</p>';
                    return;
                }
                if (!data.length) {
                    container.innerHTML = '<p>No images found.</p>';
                    return;
                }
                imagesData  = data;
                currentPage = 0;
                totalPages  = Math.ceil(imagesData.length / perPage);
                showPage();
            })
            .catch(err => {
                console.error('Error loading images:', err);
                container.innerHTML = '<p>' + err.message + '</p>';
            });
    }

    // Open the dialog when "Select Image" is clicked
    selectLink.addEventListener('click', function(e) {
        e.preventDefault();
        loadAllImages();
        showDialog();
    });

    // Close button
    if (closeBtn) {
        closeBtn.addEventListener('click', function() {
            hideDialog();
        });
    }

    // Prev/Next
    if (prevBtn) {
        prevBtn.addEventListener('click', function() {
            if (currentPage > 0) {
                currentPage--;
                showPage();
            }
        });
    }
    if (nextBtn) {
        nextBtn.addEventListener('click', function() {
            if (currentPage < totalPages - 1) {
                currentPage++;
                showPage();
            }
        });
    }
});
</script>

<div id="article-image-modal" 
     class="ui-dialog ui-corner-all ui-widget ui-widget-content ui-front txp-tagbuilder-container"
     role="dialog" aria-modal="true" aria-labelledby="article-image-modal-title"
     style="display:none;">

  <div class="ui-dialog-titlebar ui-widget-header ui-corner-all ui-helper-clearfix">
    <span class="ui-dialog-title" id="article-image-modal-title">Select an Image</span>
    <button type="button" class="ui-dialog-titlebar-close" id="close-article-image-modal" aria-label="Close">&times;</button>
  </div>

  <div class="ui-dialog-content ui-widget-content">
    <p class="help-block txp-modal-instructions">Click on a thumbnail to be inserted into the article image field.</p>
    <div id="thumbnail-container"></div>
  </div>

  <div class="ui-dialog-buttonpane ui-widget-content ui-helper-clearfix">
    <div class="ui-dialog-buttonset">
      <button id="prev-btn" class="nav-button">&#9664; Prev</button>
      <button id="next-btn" class="nav-button">Next &#9654;</button>
    </div>
  </div>
</div>
HTML;
    }

    public function fetch_images()
    {
        header('Content-Type: application/json; charset=utf-8');

        try {
            $action = gps('action');
            $id     = (int) gps('id');

            // Single image
            if ($action === 'single' && $id > 0) {
                $row = safe_row('*', 'txp_image', 'id='.$id);
                if (!$row) {
                    echo json_encode(['error' => 'Image not found']);
                    exit;
                }
                $ext  = htmlspecialchars($row['ext'], ENT_QUOTES, 'UTF-8');
                $alt  = htmlspecialchars($row['alt'], ENT_QUOTES, 'UTF-8');
                $name = htmlspecialchars($row['name'], ENT_QUOTES, 'UTF-8');
                $filename = $row['id'] . 't' . $ext;

                echo json_encode([
                    'id'            => (int)$row['id'],
                    'name'          => $name,
                    'alt'           => $alt,
                    'thumbnail_url' => hu . 'images/' . $filename
                ]);
                exit;
            }

            // All images
            $rows = safe_rows('*', 'txp_image', '1=1 ORDER BY id DESC');
            if (!$rows) {
                echo json_encode([]);
                exit;
            }

            $result = [];
            foreach ($rows as $row) {
                $imgId  = (int)$row['id'];
                $name   = htmlspecialchars($row['name'], ENT_QUOTES, 'UTF-8');
                $alt    = htmlspecialchars($row['alt'], ENT_QUOTES, 'UTF-8');
                $ext    = htmlspecialchars($row['ext'], ENT_QUOTES, 'UTF-8');
                $thumb  = $imgId . 't' . $ext;

                $result[] = [
                    'id'            => $imgId,
                    'name'          => $name,
                    'alt'           => $alt,
                    'thumbnail_url' => hu . 'images/' . $thumb
                ];
            }

            echo json_encode($result);

        } catch (Exception $e) {
            error_log('Error in ebl_article_image_modal::fetch_images: ' . $e->getMessage());
            echo json_encode(['error' => 'Unable to fetch images.']);
        }
        exit;
    }
}